<?php
/**
 * Album class for handling album-related operations
 */
class Album {
    private $db;
    
    public function __construct() {
        require_once 'database.php';
        $this->db = Database::getInstance()->getConnection();
    }
    
    /**
     * Create a new album
     * 
     * @param int $user_id User ID
     * @param string $album_name Album name
     * @param string $description Album description
     * @param bool $is_public Whether the album is public
     * @return int|bool Album ID on success, false on failure
     */
    public function createAlbum($user_id, $album_name, $description, $is_public = false) {
        try {
            $stmt = $this->db->prepare("INSERT INTO albums (user_id, album_name, description, is_public) 
                                        VALUES (:user_id, :album_name, :description, :is_public)");
            
            $stmt->bindParam(':user_id', $user_id);
            $stmt->bindParam(':album_name', $album_name);
            $stmt->bindParam(':description', $description);
            $stmt->bindParam(':is_public', $is_public, PDO::PARAM_BOOL);
            
            $stmt->execute();
            
            return $this->db->lastInsertId();
        } catch (PDOException $e) {
            error_log("Error creating album: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Get album by ID
     * 
     * @param int $album_id Album ID
     * @return array|bool Album data or false if not found
     */
    public function getAlbumById($album_id) {
        $stmt = $this->db->prepare("SELECT a.*, u.display_name as user_name, 
                                    (SELECT COUNT(*) FROM album_images WHERE album_id = a.album_id) as image_count 
                                    FROM albums a 
                                    JOIN users u ON a.user_id = u.user_id 
                                    WHERE a.album_id = :album_id");
        $stmt->bindParam(':album_id', $album_id);
        $stmt->execute();
        
        if ($stmt->rowCount() > 0) {
            return $stmt->fetch(PDO::FETCH_ASSOC);
        }
        return false;
    }
    
    /**
     * Get albums by user ID
     * 
     * @param int $user_id User ID
     * @return array Albums
     */
    public function getAlbumsByUserId($user_id) {
        $stmt = $this->db->prepare("SELECT a.*, 
                                    (SELECT COUNT(*) FROM album_images WHERE album_id = a.album_id) as image_count 
                                    FROM albums a 
                                    WHERE a.user_id = :user_id 
                                    ORDER BY a.updated_at DESC");
        $stmt->bindParam(':user_id', $user_id);
        $stmt->execute();
        
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    }
    
    /**
     * Get public albums
     * 
     * @param int $limit Maximum number of albums to return
     * @param int $offset Offset for pagination
     * @return array Albums
     */
    public function getPublicAlbums($limit = 20, $offset = 0) {
        $stmt = $this->db->prepare("SELECT a.*, u.display_name as user_name, 
                                    (SELECT COUNT(*) FROM album_images WHERE album_id = a.album_id) as image_count 
                                    FROM albums a 
                                    JOIN users u ON a.user_id = u.user_id 
                                    WHERE a.is_public = 1 
                                    ORDER BY a.updated_at DESC 
                                    LIMIT :limit OFFSET :offset");
        $stmt->bindParam(':limit', $limit, PDO::PARAM_INT);
        $stmt->bindParam(':offset', $offset, PDO::PARAM_INT);
        $stmt->execute();
        
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    }
    
    /**
     * Update album
     * 
     * @param int $album_id Album ID
     * @param string $album_name New album name
     * @param string $description New description
     * @param bool $is_public New public status
     * @return bool Success or failure
     */
    public function updateAlbum($album_id, $album_name, $description, $is_public) {
        try {
            $stmt = $this->db->prepare("UPDATE albums 
                                        SET album_name = :album_name, description = :description, is_public = :is_public 
                                        WHERE album_id = :album_id");
            
            $stmt->bindParam(':album_id', $album_id);
            $stmt->bindParam(':album_name', $album_name);
            $stmt->bindParam(':description', $description);
            $stmt->bindParam(':is_public', $is_public, PDO::PARAM_BOOL);
            
            return $stmt->execute();
        } catch (PDOException $e) {
            error_log("Error updating album: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Delete an album
     * 
     * @param int $album_id Album ID
     * @param int $user_id User ID (for security)
     * @return bool Success or failure
     */
    public function deleteAlbum($album_id, $user_id) {
        try {
            $stmt = $this->db->prepare("DELETE FROM albums WHERE album_id = :album_id AND user_id = :user_id");
            $stmt->bindParam(':album_id', $album_id);
            $stmt->bindParam(':user_id', $user_id);
            return $stmt->execute();
        } catch (PDOException $e) {
            error_log("Error deleting album: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Add image to album
     * 
     * @param int $album_id Album ID
     * @param int $image_id Image ID
     * @param int $position Position in album (optional)
     * @return bool Success or failure
     */
    public function addImageToAlbum($album_id, $image_id, $position = 0) {
        try {
            // Check if image is already in album
            $stmt = $this->db->prepare("SELECT * FROM album_images WHERE album_id = :album_id AND image_id = :image_id");
            $stmt->bindParam(':album_id', $album_id);
            $stmt->bindParam(':image_id', $image_id);
            $stmt->execute();
            
            if ($stmt->rowCount() > 0) {
                // Image already in album, update position
                $stmt = $this->db->prepare("UPDATE album_images SET position = :position WHERE album_id = :album_id AND image_id = :image_id");
            } else {
                // Add image to album
                $stmt = $this->db->prepare("INSERT INTO album_images (album_id, image_id, position) VALUES (:album_id, :image_id, :position)");
            }
            
            $stmt->bindParam(':album_id', $album_id);
            $stmt->bindParam(':image_id', $image_id);
            $stmt->bindParam(':position', $position);
            
            if ($stmt->execute()) {
                // Update album's updated_at timestamp
                $this->updateAlbumTimestamp($album_id);
                return true;
            }
            
            return false;
        } catch (PDOException $e) {
            error_log("Error adding image to album: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Remove image from album
     * 
     * @param int $album_id Album ID
     * @param int $image_id Image ID
     * @return bool Success or failure
     */
    public function removeImageFromAlbum($album_id, $image_id) {
        try {
            $stmt = $this->db->prepare("DELETE FROM album_images WHERE album_id = :album_id AND image_id = :image_id");
            $stmt->bindParam(':album_id', $album_id);
            $stmt->bindParam(':image_id', $image_id);
            
            if ($stmt->execute()) {
                // Update album's updated_at timestamp
                $this->updateAlbumTimestamp($album_id);
                return true;
            }
            
            return false;
        } catch (PDOException $e) {
            error_log("Error removing image from album: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Get images in album
     * 
     * @param int $album_id Album ID
     * @return array Images
     */
    public function getAlbumImages($album_id) {
        $stmt = $this->db->prepare("SELECT i.*, ai.position 
                                    FROM images i 
                                    JOIN album_images ai ON i.image_id = ai.image_id 
                                    WHERE ai.album_id = :album_id 
                                    ORDER BY ai.position, i.upload_date DESC");
        $stmt->bindParam(':album_id', $album_id);
        $stmt->execute();
        
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    }
    
    /**
     * Set album cover image
     * 
     * @param int $album_id Album ID
     * @param int $image_id Image ID
     * @return bool Success or failure
     */
    public function setAlbumCover($album_id, $image_id) {
        try {
            $stmt = $this->db->prepare("UPDATE albums SET cover_image_id = :image_id WHERE album_id = :album_id");
            $stmt->bindParam(':album_id', $album_id);
            $stmt->bindParam(':image_id', $image_id);
            return $stmt->execute();
        } catch (PDOException $e) {
            error_log("Error setting album cover: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Update album timestamp
     * 
     * @param int $album_id Album ID
     * @return bool Success or failure
     */
    private function updateAlbumTimestamp($album_id) {
        try {
            $stmt = $this->db->prepare("UPDATE albums SET updated_at = CURRENT_TIMESTAMP WHERE album_id = :album_id");
            $stmt->bindParam(':album_id', $album_id);
            return $stmt->execute();
        } catch (PDOException $e) {
            error_log("Error updating album timestamp: " . $e->getMessage());
            return false;
        }
    }
}
