<?php
/**
 * Image class for handling image-related operations
 */
class Image {
    private $db;
    
    public function __construct() {
        require_once 'database.php';
        $this->db = Database::getInstance()->getConnection();
    }
    
    /**
     * Upload a new image
     * 
     * @param int $user_id User ID
     * @param array $file $_FILES array element
     * @param string $title Image title
     * @param string $description Image description
     * @param bool $is_public Whether the image is public
     * @return int|bool Image ID on success, false on failure
     */
    public function uploadImage($user_id, $file, $title, $description, $is_public = false) {
        // Validate file
        $allowed_types = ['image/jpeg', 'image/png', 'image/gif'];
        if (!in_array($file['type'], $allowed_types)) {
            return false;
        }
        
        // Generate unique filename
        $file_extension = pathinfo($file['name'], PATHINFO_EXTENSION);
        $new_filename = uniqid('img_') . '.' . $file_extension;
        $upload_path = UPLOAD_DIR . $new_filename;
        
        // Move uploaded file
        if (!move_uploaded_file($file['tmp_name'], $upload_path)) {
            return false;
        }
        
        // Create thumbnail
        $thumbnail_filename = 'thumb_' . $new_filename;
        $thumbnail_path = THUMBNAIL_DIR . $thumbnail_filename;
        $this->createThumbnail($upload_path, $thumbnail_path, 300);
        
        // Get image dimensions
        $image_info = getimagesize($upload_path);
        $width = $image_info[0];
        $height = $image_info[1];
        
        // Insert into database
        try {
            $stmt = $this->db->prepare("INSERT INTO images (user_id, file_name, file_path, thumbnail_path, title, description, file_size, file_type, width, height, is_public) 
                                        VALUES (:user_id, :file_name, :file_path, :thumbnail_path, :title, :description, :file_size, :file_type, :width, :height, :is_public)");
            
            $file_path = 'uploads/' . $new_filename;
            $thumbnail_path = 'uploads/thumbnails/' . $thumbnail_filename;
            
            $stmt->bindParam(':user_id', $user_id);
            $stmt->bindParam(':file_name', $file['name']);
            $stmt->bindParam(':file_path', $file_path);
            $stmt->bindParam(':thumbnail_path', $thumbnail_path);
            $stmt->bindParam(':title', $title);
            $stmt->bindParam(':description', $description);
            $stmt->bindParam(':file_size', $file['size']);
            $stmt->bindParam(':file_type', $file['type']);
            $stmt->bindParam(':width', $width);
            $stmt->bindParam(':height', $height);
            $stmt->bindParam(':is_public', $is_public, PDO::PARAM_BOOL);
            
            $stmt->execute();
            
            return $this->db->lastInsertId();
        } catch (PDOException $e) {
            error_log("Error uploading image: " . $e->getMessage());
            // Delete uploaded files if database insert fails
            @unlink($upload_path);
            @unlink($thumbnail_path);
            return false;
        }
    }
    
    /**
     * Create a thumbnail from an image
     * 
     * @param string $source_path Source image path
     * @param string $target_path Target thumbnail path
     * @param int $max_dimension Maximum width or height
     * @return bool Success or failure
     */
    private function createThumbnail($source_path, $target_path, $max_dimension = 300) {
        // Get image type
        $image_info = getimagesize($source_path);
        $image_type = $image_info[2];
        
        // Load the image based on its type
        switch ($image_type) {
            case IMAGETYPE_JPEG:
                $source_image = imagecreatefromjpeg($source_path);
                break;
            case IMAGETYPE_PNG:
                $source_image = imagecreatefrompng($source_path);
                break;
            case IMAGETYPE_GIF:
                $source_image = imagecreatefromgif($source_path);
                break;
            default:
                return false;
        }
        
        // Get original dimensions
        $width = imagesx($source_image);
        $height = imagesy($source_image);
        
        // Calculate new dimensions
        if ($width > $height) {
            $new_width = $max_dimension;
            $new_height = floor($height * ($max_dimension / $width));
        } else {
            $new_height = $max_dimension;
            $new_width = floor($width * ($max_dimension / $height));
        }
        
        // Create thumbnail image
        $thumbnail = imagecreatetruecolor($new_width, $new_height);
        
        // Preserve transparency for PNG and GIF
        if ($image_type == IMAGETYPE_PNG || $image_type == IMAGETYPE_GIF) {
            imagealphablending($thumbnail, false);
            imagesavealpha($thumbnail, true);
            $transparent = imagecolorallocatealpha($thumbnail, 255, 255, 255, 127);
            imagefilledrectangle($thumbnail, 0, 0, $new_width, $new_height, $transparent);
        }
        
        // Resize image
        imagecopyresampled($thumbnail, $source_image, 0, 0, 0, 0, $new_width, $new_height, $width, $height);
        
        // Save thumbnail
        switch ($image_type) {
            case IMAGETYPE_JPEG:
                imagejpeg($thumbnail, $target_path, 90);
                break;
            case IMAGETYPE_PNG:
                imagepng($thumbnail, $target_path);
                break;
            case IMAGETYPE_GIF:
                imagegif($thumbnail, $target_path);
                break;
        }
        
        // Free memory
        imagedestroy($source_image);
        imagedestroy($thumbnail);
        
        return true;
    }
    
    /**
     * Get image by ID
     * 
     * @param int $image_id Image ID
     * @return array|bool Image data or false if not found
     */
    public function getImageById($image_id) {
        $stmt = $this->db->prepare("SELECT i.*, u.display_name as user_name 
                                    FROM images i 
                                    JOIN users u ON i.user_id = u.user_id 
                                    WHERE i.image_id = :image_id");
        $stmt->bindParam(':image_id', $image_id);
        $stmt->execute();
        
        if ($stmt->rowCount() > 0) {
            return $stmt->fetch(PDO::FETCH_ASSOC);
        }
        return false;
    }
    
    /**
     * Get images by user ID
     * 
     * @param int $user_id User ID
     * @param int $limit Maximum number of images to return
     * @param int $offset Offset for pagination
     * @return array Images
     */
    public function getImagesByUserId($user_id, $limit = 20, $offset = 0) {
        $stmt = $this->db->prepare("SELECT * FROM images 
                                    WHERE user_id = :user_id 
                                    ORDER BY upload_date DESC 
                                    LIMIT :limit OFFSET :offset");
        $stmt->bindParam(':user_id', $user_id);
        $stmt->bindParam(':limit', $limit, PDO::PARAM_INT);
        $stmt->bindParam(':offset', $offset, PDO::PARAM_INT);
        $stmt->execute();
        
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    }
    
    /**
     * Get public images
     * 
     * @param int $limit Maximum number of images to return
     * @param int $offset Offset for pagination
     * @return array Images
     */
    public function getPublicImages($limit = 20, $offset = 0) {
        $stmt = $this->db->prepare("SELECT i.*, u.display_name as user_name 
                                    FROM images i 
                                    JOIN users u ON i.user_id = u.user_id 
                                    WHERE i.is_public = 1 
                                    ORDER BY i.upload_date DESC 
                                    LIMIT :limit OFFSET :offset");
        $stmt->bindParam(':limit', $limit, PDO::PARAM_INT);
        $stmt->bindParam(':offset', $offset, PDO::PARAM_INT);
        $stmt->execute();
        
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    }
    
    /**
     * Update image metadata
     * 
     * @param int $image_id Image ID
     * @param string $title New title
     * @param string $description New description
     * @param bool $is_public New public status
     * @return bool Success or failure
     */
    public function updateImage($image_id, $title, $description, $is_public) {
        try {
            $stmt = $this->db->prepare("UPDATE images 
                                        SET title = :title, description = :description, is_public = :is_public 
                                        WHERE image_id = :image_id");
            
            $stmt->bindParam(':image_id', $image_id);
            $stmt->bindParam(':title', $title);
            $stmt->bindParam(':description', $description);
            $stmt->bindParam(':is_public', $is_public, PDO::PARAM_BOOL);
            
            return $stmt->execute();
        } catch (PDOException $e) {
            error_log("Error updating image: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Delete an image
     * 
     * @param int $image_id Image ID
     * @param int $user_id User ID (for security)
     * @return bool Success or failure
     */
    public function deleteImage($image_id, $user_id) {
        try {
            // Get image data first to delete files
            $stmt = $this->db->prepare("SELECT file_path, thumbnail_path FROM images 
                                        WHERE image_id = :image_id AND user_id = :user_id");
            $stmt->bindParam(':image_id', $image_id);
            $stmt->bindParam(':user_id', $user_id);
            $stmt->execute();
            
            if ($stmt->rowCount() == 0) {
                return false; // Image not found or not owned by user
            }
            
            $image = $stmt->fetch(PDO::FETCH_ASSOC);
            
            // Delete from database
            $stmt = $this->db->prepare("DELETE FROM images WHERE image_id = :image_id AND user_id = :user_id");
            $stmt->bindParam(':image_id', $image_id);
            $stmt->bindParam(':user_id', $user_id);
            $result = $stmt->execute();
            
            if ($result) {
                // Delete files
                @unlink(UPLOAD_DIR . basename($image['file_path']));
                @unlink(THUMBNAIL_DIR . basename($image['thumbnail_path']));
                return true;
            }
            
            return false;
        } catch (PDOException $e) {
            error_log("Error deleting image: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Search images by title or description
     * 
     * @param string $search_term Search term
     * @param int $user_id Optional user ID to limit search to user's images
     * @param int $limit Maximum number of images to return
     * @param int $offset Offset for pagination
     * @return array Images
     */
    public function searchImages($search_term, $user_id = null, $limit = 20, $offset = 0) {
        $search_term = '%' . $search_term . '%';
        
        if ($user_id) {
            $stmt = $this->db->prepare("SELECT * FROM images 
                                        WHERE user_id = :user_id 
                                        AND (title LIKE :search_term OR description LIKE :search_term) 
                                        ORDER BY upload_date DESC 
                                        LIMIT :limit OFFSET :offset");
            $stmt->bindParam(':user_id', $user_id);
        } else {
            $stmt = $this->db->prepare("SELECT i.*, u.display_name as user_name 
                                        FROM images i 
                                        JOIN users u ON i.user_id = u.user_id 
                                        WHERE i.is_public = 1 
                                        AND (i.title LIKE :search_term OR i.description LIKE :search_term) 
                                        ORDER BY i.upload_date DESC 
                                        LIMIT :limit OFFSET :offset");
        }
        
        $stmt->bindParam(':search_term', $search_term);
        $stmt->bindParam(':limit', $limit, PDO::PARAM_INT);
        $stmt->bindParam(':offset', $offset, PDO::PARAM_INT);
        $stmt->execute();
        
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    }
}
