<?php
/**
 * User class for handling user-related operations
 */
class User {
    private $db;
    private $user_id;
    private $google_id;
    private $email;
    private $display_name;
    private $profile_picture;
    private $is_admin;
    
    public function __construct() {
        require_once 'database.php';
        $this->db = Database::getInstance()->getConnection();
    }
    
    /**
     * Get user by Google ID
     */
    public function getUserByGoogleId($google_id) {
        $stmt = $this->db->prepare("SELECT * FROM users WHERE google_id = :google_id");
        $stmt->bindParam(':google_id', $google_id);
        $stmt->execute();
        
        if ($stmt->rowCount() > 0) {
            $user = $stmt->fetch(PDO::FETCH_ASSOC);
            $this->setUserData($user);
            return true;
        }
        return false;
    }
    
    /**
     * Create new user from Google data
     */
    public function createUserFromGoogle($google_data) {
        try {
            $stmt = $this->db->prepare("INSERT INTO users (google_id, email, display_name, profile_picture) 
                                        VALUES (:google_id, :email, :display_name, :profile_picture)");
            
            $stmt->bindParam(':google_id', $google_data['google_id']);
            $stmt->bindParam(':email', $google_data['email']);
            $stmt->bindParam(':display_name', $google_data['display_name']);
            $stmt->bindParam(':profile_picture', $google_data['profile_picture']);
            
            $stmt->execute();
            
            $this->user_id = $this->db->lastInsertId();
            $this->google_id = $google_data['google_id'];
            $this->email = $google_data['email'];
            $this->display_name = $google_data['display_name'];
            $this->profile_picture = $google_data['profile_picture'];
            $this->is_admin = false;
            
            return true;
        } catch (PDOException $e) {
            error_log("Error creating user: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Update user login timestamp
     */
    public function updateLastLogin() {
        if (!$this->user_id) return false;
        
        $stmt = $this->db->prepare("UPDATE users SET last_login = CURRENT_TIMESTAMP WHERE user_id = :user_id");
        $stmt->bindParam(':user_id', $this->user_id);
        return $stmt->execute();
    }
    
    /**
     * Create a new session for the user
     */
    public function createSession() {
        if (!$this->user_id) return false;
        
        $session_id = bin2hex(random_bytes(32));
        $expires_at = date('Y-m-d H:i:s', time() + SESSION_LIFETIME);
        $ip_address = $_SERVER['REMOTE_ADDR'];
        $user_agent = $_SERVER['HTTP_USER_AGENT'];
        
        try {
            $stmt = $this->db->prepare("INSERT INTO user_sessions (session_id, user_id, expires_at, ip_address, user_agent) 
                                        VALUES (:session_id, :user_id, :expires_at, :ip_address, :user_agent)");
            
            $stmt->bindParam(':session_id', $session_id);
            $stmt->bindParam(':user_id', $this->user_id);
            $stmt->bindParam(':expires_at', $expires_at);
            $stmt->bindParam(':ip_address', $ip_address);
            $stmt->bindParam(':user_agent', $user_agent);
            
            $stmt->execute();
            
            return $session_id;
        } catch (PDOException $e) {
            error_log("Error creating session: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Validate a session
     */
    public function validateSession($session_id) {
        $stmt = $this->db->prepare("SELECT u.* FROM users u 
                                    JOIN user_sessions s ON u.user_id = s.user_id 
                                    WHERE s.session_id = :session_id AND s.expires_at > NOW()");
        $stmt->bindParam(':session_id', $session_id);
        $stmt->execute();
        
        if ($stmt->rowCount() > 0) {
            $user = $stmt->fetch(PDO::FETCH_ASSOC);
            $this->setUserData($user);
            return true;
        }
        return false;
    }
    
    /**
     * Delete a session
     */
    public function deleteSession($session_id) {
        $stmt = $this->db->prepare("DELETE FROM user_sessions WHERE session_id = :session_id");
        $stmt->bindParam(':session_id', $session_id);
        return $stmt->execute();
    }
    
    /**
     * Set user data from database result
     */
    private function setUserData($user) {
        $this->user_id = $user['user_id'];
        $this->google_id = $user['google_id'];
        $this->email = $user['email'];
        $this->display_name = $user['display_name'];
        $this->profile_picture = $user['profile_picture'];
        $this->is_admin = $user['is_admin'];
    }
    
    // Getters
    public function getUserId() {
        return $this->user_id;
    }
    
    public function getEmail() {
        return $this->email;
    }
    
    public function getDisplayName() {
        return $this->display_name;
    }
    
    public function getProfilePicture() {
        return $this->profile_picture;
    }
    
    public function isAdmin() {
        return $this->is_admin;
    }
}
